@@ -1,8 +1,8 @@
-from .matrices import GaussElimination,CramerRule,Matrix,Identity
+from .matrices import CramerRule,Matrix,Identity
 from  .vectors import scalarproduct,orthogonalVectors,indexPivot,colinear,Vector
 
 __all__=['Vector',
 'scalarproduct','orthogonalVectors','indexPivot','colinear','Matrix',
-'Identity','GaussElimination','CramerRule']
+'Identity','CramerRule']
 
 
