@@ -1,6 +1,6 @@
 
 import math
-from LinAlgebraPy.vectors import vector
+
 from LinAlgebraPy.vectors.vector import Vector, colinear,indexPivot, indexPivotColumn
 from LinAlgebraPy.matrices.matrixfunc import onlytypeM,sortList,sortMatrix
 from math import sqrt
@@ -56,8 +56,10 @@ class Matrix():
         onlylist=onlytypeM(args,list)
         onlyvector=onlytypeM(args,Vector)
         onlytuple=onlytypeM(args,tuple)
+        types=[float,int,complex,list,tuple,Vector]
         length=len(args)
         i=0
+        
         if len(args)==2 and type(args[0])==int and type(args[1])==int:
             (m,n)=args
             new=[]
@@ -71,7 +73,7 @@ class Matrix():
                 i+=1
                 j=0        
         elif onlylist:
-            if type(args[0][0])==int:
+            if type(args[0][0]) in types[0:3]:
                 while i<length:
                     c=args[i]
                     elementsM.append(c)
@@ -94,7 +96,9 @@ class Matrix():
                     (m,n)=ArgsIn[0].dim
                     while i<m:
                         elementsM+=[[j.elements[i][0] for j in ArgsIn ]]
-                        i+=1
+                        i+=1     
+            else:
+                raise ValueError('Type of Arguments allowed are in this list => [float,int,complex,list,Vector]')
         elif onlytuple:
             while i<length:
                 c=args[i]
@@ -114,8 +118,29 @@ class Matrix():
             raise TypeError("Arguments allowed are list, tuple and Vector ")
         self.elements=elementsM
         self.dim=(len(self.elements),len(self.elements[0]))
+        for  i in range(self.dim[0]):
+            for j in range(self.dim[1]):
+                elem=elementsM[i][j]
+                if type(elem) in types[0:3]:
+                    instance=type(elem)  
+                    if instance!=complex and elem<=1e-13 and elem>=-1e-13:  #changed type in echelon with N and Echelon
+                        elementsM[i][j]=0
+                    elif instance==complex:
+                        real=complex(elementsM[i][j]).real
+                        imag=complex(elementsM[i][j]).imag
+                        comparereal=(real<=1e-13 and real>=-1e-13) 
+                        compareimag=(imag<=1e-13 and imag>=-1e-13)
+                        if comparereal and compareimag:
+                            elementsM[i][j]=0;
+                        elif compareimag:
+                            elementsM[i][j]=round(real,14)
+                        elif comparereal:
+                            elementsM[i][j]=round(imag,14)*1j
+                else:
+                    raise ValueError("1-Arguments allowed are list, tuple , int, complex, float and Vector \n 2-You can't combine Vector , tuple and list together . It should be unique")
+                
         self.isSquared=self.dim[0]==self.dim[1]
-
+        
     def __add__(self,mat):
         """Adding a Matrix
 
@@ -229,8 +254,25 @@ class Matrix():
             return Vector(m.elements[j]).transpose()
         else:
             raise ValueError()
+
     def __setitem__(self, index, val):
         (i,j)=index
+        dimension=self.dim
+        instance=type(val)          
+        if instance!=complex and val<=1e-14 and val>=-1e-14:  #changed type in echelon with N and Echelon
+            val=0
+        elif instance==complex:
+            real=complex(val).real
+            imag=complex(val).imag
+            comparereal=(real<=1e-13 and real>=-1e-13) 
+            compareimag=(imag<=1e-13 and imag>=-1e-13)
+            if comparereal and compareimag:
+                val=0;
+            elif compareimag:
+                val=round(real,14)
+            elif comparereal:
+                val=round(imag,14)*1j
+                
         if type(i)==int:
             #Format M[i,j]= integer
             self.elements[i][j]=val
@@ -269,7 +311,7 @@ class Matrix():
  
     def echelon(self,choice=None):
         """Echelon a Matrix
-        This function has a float precision to e-14 for better performance
+        This function has a float precision to 1e-14 and 1e14 for better performance
         Args:
             choice (string, optional): [description]. Defaults to None.
 
@@ -285,6 +327,7 @@ class Matrix():
         for i in range(len(index)-k):
             (index,Vect,k,neg)=sortList(index,Vect)
             ind=index[i]
+
             for j in range(i+1,len(index)-k):
                 if index[j] in L:
                     break
@@ -292,15 +335,30 @@ class Matrix():
                     Vect[j]= Vect[j] - Vect[i]*(Vect[j][ind]/Vect[i][ind])
             L.append(ind)
             index=[indexPivot(x) for x in Vect]
-
+        
         if choice=="DET":
             return (Matrix(Vect),neg)
-        
+        """        #testing here Precision #1 
         for  i in range(m):
             for j in range(n):
-                if Vect[i][j]<=1e-14 and Vect[i][j]>=-1e-14:
+                elem=Vect[i][j]
+                instance=type(elem)  
+                if instance!=complex and elem<=1e-14 and elem>=-1e-14:  #changed type in echelon with N and Echelon
                     Vect[i][j]=0
-      
+                    
+                elif instance==complex:
+                    real=complex(Vect[i][j]).real
+                    imag=complex(Vect[i][j]).imag
+                    comparereal=(real<=1e-14 and real>=-1e-14) 
+                    compareimag=(imag<=1e-14 and imag>=-1e-14)
+                    if comparereal and compareimag:
+                        Vect[i][j]=0;
+                    elif compareimag:
+                        Vect[i][j]=round(real,14)
+                    elif comparereal:
+                        Vect[i][j]=round(imag,14)
+       """ #testing here Precision #1 
+
                       
         index=[indexPivot(x) for x in Vect]
         for i in range(len(index)):
@@ -309,7 +367,6 @@ class Matrix():
             else:
                 Vect[i]=Vect[i]*(1/Vect[i][index[i]])
                 
-
         if choice==None or choice=="REF":
             return Matrix(Vect)
 
@@ -319,6 +376,7 @@ class Matrix():
                     break
                 else:
                     Vect[i]=Vect[i]-(Vect[j]*Vect[i][index[j]])
+        
         if choice=="RREF":
             return Matrix(Vect)
 
@@ -349,7 +407,7 @@ class Matrix():
                 e=elem[i][j]
                 if type(e)!= complex:
                     e=round(e,4)
-                string+=" {:^4g} ".format(e)
+                string+=" {:^g} ".format(e)
                 j+=1
             string+=') \n'
             j=0
@@ -382,37 +440,41 @@ class Matrix():
         index=[indexPivot(x) for x in Vect]
         (index,Vect,k,neg,Nvect)=sortMatrix(index,Vect,Nvect)
         L=[]
-        
+       
         for i in range(len(index)-k):
             (index,Vect,k,neg,Nvect)=sortMatrix(index,Vect,Nvect)
             ind=index[i]
-            for  i in range(m):
-                for j in range(n):
-                    if Vect[i][j]<=1e-14 and Vect[i][j]>=-1e-14:
-                        Vect[i][j]=0
             if index[i]==-1:
                 break
             else:
+                #here there was a problem on 06/09/2021
+                
+               
+
                 Nvect[i]=Nvect[i]*(1/Vect[i][ind])
                 Vect[i]=Vect[i]*(1/Vect[i][ind])
-
             for j in range(i+1,len(index)-k):
+                
                 if index[j] in L:
                     break
                 if index[i]==index[j]:
+                    
+                   
                     Nvect[j]= Nvect[j] - Nvect[i]*(Vect[j][ind])
                     Vect[j]= Vect[j] - Vect[i]*(Vect[j][ind])
+                
             L.append(ind)
             index=[indexPivot(x) for x in Vect]
-             
+
         for j in range(len(index)):
             for i in range(j):
                 if i==-1 or Vect[i].isNul():
                     break
                 else:
+                    
                     Nvect[i]=Nvect[i]-(Nvect[j]*Vect[i][index[j]])
-
                     Vect[i]=Vect[i]-(Vect[j]*Vect[i][index[j]])
+        
         if n1==1:
             return Matrix(Nvect).transpose()
         else:
@@ -462,7 +524,11 @@ class Matrix():
         if self._DiagonalMatrix():
             (m1,n1)=self.dim
             for i in range(m1):
-               self.elements[i][i]**=n
+                if n<0 and self.elements[i][i]==0:
+                     raise ValueError('The matrix is not Inversible and can not accept negative exponents \n Reminder : if an eigenvalue is equal to 0, the matrix is not inversible ')
+                else:
+                   self.elements[i][i]**=n 
+                
             return Matrix(self.elements)
         else:
             raise ValueError('Matrix should be squared and diagonal')
@@ -491,10 +557,7 @@ class Matrix():
             if index[i]==max and index[i]!=-1 or index[i] in indexCroissant:
                 vect1=idenVectors[i]
                 for j in range(m):
-                    vect1[j]-=Vect[i][j]
-                    
-
-               
+                    vect1[j]-=Vect[i][j]               
                 NullSpace.append(vect1)
             elif index[i]==-1 :
                 NullSpace.append(idenVectors[i]);   
@@ -520,13 +583,17 @@ class Matrix():
             det*=neg
             if m==2 and n==2:
                 delta=((trace**2)-4*det)
-                if delta<0:
-                    eigenV.extend([(-trace-delta**.5)/2,(-trace+delta**.5)/2])
-                elif delta>0:
+                
+                if type(delta)!=complex:
+                    if  delta<0:
+                        eigenV.extend([(-trace-1j*sqrt(-delta))/2,(-trace+1j*sqrt(-delta))/2])
+                    elif delta>0:
 
-                    eigenV.extend([(-trace-delta**.5)/2,(-trace+delta**.5)/2])
+                        eigenV.extend([(-trace-sqrt(delta))/2,(-trace+sqrt(delta))/2])
+                    else:
+                        eigenV.append(-trace/2)
                 else:
-                    eigenV.append(-trace/2)
+                    raise ValueError("Can't compute eigenvalues with complex determinant yet! :(")
             return eigenV
 
         else:
@@ -543,28 +610,31 @@ class Matrix():
         else:
             raise ValueError('The matrix should be squared')
     def __pow__(self,var):
-        if(self.Diagonalizable()): 
-            (m,n)=self.dim
-            diagonal=Identity(m)
-            eigenValues=self.eigenvalues()
-            null=[]
-            for i in range(m):
-                diagonal[i,i]*=eigenValues[i]
-            for i in range(len(eigenValues)):
-                null+=self.Null(sub=eigenValues[i])
-            eigenVectorsMatrix=Matrix(null)
-
-            inverseEigenVectorsMatrix=eigenVectorsMatrix.__invert__()
-            return eigenVectorsMatrix*diagonal._expDiagonalMatrix(var)*inverseEigenVectorsMatrix
+        (m,n)=self.dim
+        if m==2 and n==2:
+            if(self.Diagonalizable()): 
+                (m,n)=self.dim
+                diagonal=Identity(m)
+                eigenValues=self.eigenvalues()
+                null=[]
+                for i in range(m):
+                    diagonal[i,i]*=eigenValues[i]
+                for i in range(len(eigenValues)):
+                    null+=self.Null(sub=eigenValues[i])
+                eigenVectorsMatrix=Matrix(null)
+                inverseEigenVectorsMatrix=eigenVectorsMatrix.__invert__()
+                return eigenVectorsMatrix*diagonal._expDiagonalMatrix(var)*inverseEigenVectorsMatrix
+        else:
+            raise ValueError("This operation only works for 2x2 real matrix for now")
     def __ipow__(self,var):
         return self.__pow__(var)
     
         
-def GaussElimination(ma,v1):
-    return ma.echelonWithN(v1)
+#def GaussElimination(ma,v1) -> Matrix:
+#    return ma.echelonWithN(v1)
 
 
-def CramerRule(ma,v1):
+def CramerRule(ma,v1) -> Vector:
     (m,n)=ma.dim
     ele=[]
     determ=ma.det()
@@ -576,3 +646,5 @@ def CramerRule(ma,v1):
     return Vector(ele)
 
 
+
+
